/**
 * @file multiKP.cpp
 * @author Luigi De Giovanni
 * @brief multiKP example implementation
 */

#include "multiKP.hpp"
#include <sstream>
#include "scip_exception.hpp"

using namespace std;
using namespace scipMeMOC;

/* constructor */
scipMeMOC::MultiKPSolver::MultiKPSolver(char * fname)
   : _scip(0), _consCap()
{
cout << "READING... " << endl;
   // initialize instance data
   readData(fname);
   
cout << "INIT SCIP... " << endl;
   // initialize scip
   SCIP_CALL_EXC( SCIPcreate(& _scip) );

   // load default plugins like separators, heuristics, etc.
   SCIP_CALL_EXC( SCIPincludeDefaultPlugins(_scip) );

   // disable scip output to stdout
   SCIP_CALL_EXC( SCIPsetMessagehdlr(NULL) );

   // create an empty problem
   SCIP_CALL_EXC( SCIPcreateProb(_scip, "multiKP", NULL, NULL, NULL, NULL, NULL, NULL) );

   // set the objective sense to maximize, default is minimize
   SCIP_CALL_EXC( SCIPsetObjsense(_scip, SCIP_OBJSENSE_MAXIMIZE) );

   // create a binary variable for every item
   ostringstream namebuf;
   for(int j = 0; j < _numItem; ++j)
   {
       SCIP_VAR * var;
       namebuf.str("");
       namebuf << "x#" << j;
       
       // create the SCIP_VAR object
       SCIP_CALL_EXC( SCIPcreateVar(_scip, & var, namebuf.str().c_str(), 0.0, 1.0, _profit[j], SCIP_VARTYPE_BINARY, TRUE, FALSE, NULL, NULL, NULL, NULL) );
       
       // add the SCIP_VAR object to the scip problem
       SCIP_CALL_EXC( SCIPaddVar(_scip, var) );
       
       // storing the SCIP_VAR pointer for later access
       _xload.push_back(var);
   }

   // create constraints
   // one queen per row
   for (int k = 0; k < _numDim; ++k)
   {
      SCIP_CONS * cons;
      namebuf.str("");
      namebuf<<"dim_"<<k;

      // create SCIP_CONS object
      SCIP_CALL_EXC( SCIPcreateConsLinear(_scip, & cons, namebuf.str().c_str(), 0, NULL, NULL, -SCIPinfinity(_scip), double(_capacity[k]),
					  TRUE, TRUE, TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE) );

      // add the vars belonging to field in this row to the constraint
      for (int j = 0; j < _numItem; ++j)
         SCIP_CALL_EXC( SCIPaddCoefLinear(_scip, cons, _xload[j], _weight[k][j]) );

      // add the constraint to scip
      SCIP_CALL_EXC( SCIPaddCons(_scip, cons) );

      // store the constraint for later on
      _consCap.push_back(cons);
   }

}


/* display the solution */
void scipMeMOC::MultiKPSolver::display(std::ostream& out)
{
   // get the best found solution from scip
   SCIP_SOL * sol = SCIPgetBestSol(_scip);
   // when SCIP did not succeed then sol is NULL
   if (sol == NULL)
   {
      out << "no solution found" << endl;
      return;
   }
   
   out << "solution value: " << SCIPgetSolOrigObj(_scip, sol) << endl;

   out << "loaded items (.# index (name), dim , profit):" << endl;
   int n = 0;
   for (int j = 0; j < _numItem; ++j) {
       if ( SCIPgetSolVal(_scip, sol, _xload[j]) > 0.5 ) {
           out << ++n << "#\t" << j << " (" << _xload[j]->name << "),\t";
           for ( int k=0; k < _numDim-1; k++ ){
               out << _weight[k][j] << " x "; 
           }
           out << _weight[_numDim-1][j] << " ,\t" << _profit[j] << endl;
       }
   }
   out << endl;
}


/* destructor */
scipMeMOC::MultiKPSolver::~MultiKPSolver(void)
{
cout << "DESTROYING... " << endl;
   // since the SCIPcreateVar captures all variables, we have to release them now
   for (int j = 0; j < _numItem; ++j)
   {
       SCIP_CALL_EXC( SCIPreleaseVar(_scip, & _xload[j]) );
   }
   _xload.clear();

   // the same for the constraints
   for (vector<SCIP_CONS *>::size_type k = 0; k < _consCap.size(); ++k)
      SCIP_CALL_EXC( SCIPreleaseCons(_scip, &_consCap[k]) );
   _consCap.clear();

   // after releasing all vars and cons we can free the scip problem
   // remember this has allways to be the last call to scip
   SCIP_CALL_EXC( SCIPfree( & _scip) );
   
   _capacity.clear();
   _profit.clear();
   _weight.clear();
}

/* solve the multiKP problem */
void scipMeMOC::MultiKPSolver::solve(void)
{
cout << "SOLVING... " << endl;
   // this tells scip to start the solution process
   SCIP_CALL_EXC( SCIPsolve(_scip) );
}


int scipMeMOC::MultiKPSolver::readData(char * fname) {
   int k, j, intBuf;
   FILE * fpdata = fopen(fname,"r");//   int n = abs(atoi(argv[1]));
   if ( fpdata == NULL ) {
       cout << "ERROR reading " << fname << endl;
       exit(-1);
   }
   fscanf(fpdata,"numDim:%d\n",&_numDim);
   fscanf(fpdata,"numItem:%d\n",&_numItem);   
   
   _capacity.reserve(_numDim);
   fscanf(fpdata,"Capacities:\n");
   for (k = 0; k<_numDim; k++) {
       fscanf(fpdata,"%d\n",&intBuf);
       _capacity.push_back(intBuf);
   }
   _profit.reserve(_numItem);
   fscanf(fpdata,"Profits:\n");
   for (j = 0; j<_numItem; j++) {
       fscanf(fpdata,"%d\n",&intBuf);
       _profit.push_back(intBuf);
   }   
   
   fscanf(fpdata,"Weights:\n");
   _weight.reserve(_numDim);
   for (k = 0; k<_numDim; k++) {  
       _weight.push_back(vector<int>());
       for (j = 0; j<_numItem; j++) {
           fscanf(fpdata,"%d",&intBuf);
           _weight[k].push_back(intBuf);
       }
   }
   fclose(fpdata);
   
   return _numDim * _numItem;
}
