/**@file  multiKP.hpp
 * @brief  multiKP example
 * @author Luigi De Giovanni
 */

#ifndef MULTIKP_H
#define MULTIKP_H

#include <vector>
#include <iostream>

#include <scip/scip.h>
#include <scip/scipdefplugins.h>

namespace scipMeMOC
{
   /**@class MultiKPSolver
    * @brief solver class for the multi-dimensional knapsack problem
    *
    *  this class implements a solver for the multi-dimensional knapsack problem as an mip model, which will be solved using SCIP
    */
   class MultiKPSolver
   {
   private:

      /** @brief pointer to scip structure */
      SCIP * _scip;

      /** @brief number of dimensions  */
      int _numDim;
      /** @brief capacity per dimension  */
      std::vector<int> _capacity;

      /** @brief number of items  */
      int _numItem;
      /** @brief profit per item  */
      std::vector<int> _profit;
      /** @brief weight per dimension per item */         
      std::vector<std::vector<int> > _weight;
      
      /** @brief one binary variable for each item */
      std::vector<SCIP_VAR *> _xload;

      /** @brief constraints for knapsasck capacities */
      std::vector<SCIP_CONS *> _consCap;

   public:
      /** @brief constructs the integer programming model for the multi-KP problem
       *
       * @param[in] fname the name of the file containing the instance
       */
      MultiKPSolver(char * fname );

      /** @brief destructor */
      ~MultiKPSolver();

      void solve(void); ///< solves the multi-KP problem using SCIPsolve

      /** @brief display the solution
       *
       * a simple ASCII output function to display the solution of the multi-KP problem
       * @param[in,out] out ostream class for output(default cout)
       */
      void display(std::ostream & out = std::cout);
      
   private:
   int readData ( char * fname );
   };
}
#endif

